""" Tool to manipulate timers """
import toml

from typing import Optional, Union, Dict

from pathlib import Path
from datetime import datetime, timedelta

from pomme.duration import Duration


# TODO: don't use direct access but only write logic here
# - remove 'database' attribute and only keep caracteristics of timer
# - import timer from another file (containing the database access logic)


class Timer:
    """ manipulate timer """

    database: Dict[str, Union[None, int, float, str, datetime]] = {
        "start_time": None,
        "timedelta": None,
    }

    def __init__(self, database_location: str, *args, **kwargs):
        self.database_path = Path(database_location).expanduser()
        if self.database_path.is_dir():
            raise IOError("The database location is a directory")
        if not self.database_path.is_file():
            dp = self.database_path
            dp.parent.mkdir(parents=True, exist_ok=True)
            self.update_database()
            print("Database created at {}".format(dp))

        self.database = dict(toml.load(self.database_path))

    def update_database(self):
        with open(self.database_path, "w") as f:
            toml.dump(self.database, f)

    @property
    def begin_time(self) -> Optional[datetime]:
        if "begin_time" in self.database and isinstance(
            self.database["begin_time"], datetime
        ):
            return self.database["begin_time"]
        else:
            return None

    @begin_time.setter
    def begin_time(self, begin_time: datetime) -> None:
        self.database["begin_time"] = begin_time
        self.update_database()

    @property
    def duration(self) -> Optional[timedelta]:
        if "duration" in self.database and isinstance(
            self.database["duration"], float
        ):
            return timedelta(seconds=self.database["duration"])
        else:
            return None

    @duration.setter
    def duration(self, duration: Duration) -> None:
        self.database["duration"] = duration.total_seconds()
        self.update_database()

    def start(self, duration: str) -> None:
        self.begin_time = datetime.utcnow()
        self.duration = Duration.from_str(duration)

    def cancel(self) -> None:
        self.begin_time = None

    def running(self) -> Union[bool, None]:
        """ is the timer running ? """
        begin_time, duration = self.begin_time, self.duration
        if begin_time is None or duration is None:
            return None
        else:
            return begin_time + duration > datetime.utcnow()

    def remaining(self) -> Union[str, None]:
        """ time remaining for the timer """
        begin_time, duration = self.begin_time, self.duration
        if begin_time is None or duration is None:
            return None
        else:
            time = begin_time + duration - datetime.utcnow()
            aproximate_time = timedelta(seconds=round(time.total_seconds()))
            return str(aproximate_time)
